// ==========================================
//             Detect changes
// ==========================================

function ContenteditableChange() {
    // Add change trigger to contenteditable elements
    $('body').on('focus', '[contenteditable]', function() {
        var $this = $(this);
        $this.data('before', $this.html());
        return $this;
    }).on('blur keyup paste input', '[contenteditable]', function() {
        var $this = $(this);
        if ($this.data('before') !== $this.html()) {
            $this.data('before', $this.html());
            $this.trigger('change');
        }
        return $this;
    });
};

function initializeDectectChanges() {
    ContenteditableChange();

    // Fix line breaks in table cells
    $("td").blur(function() {
        content = $(this).html();
        content = content.replace(/<br>/gi, "[br]"); // Line breaks
        content = content.replace(/\n/g, ""); // Line breaks
        $(this).html(content);
        content = $(this).text().replace(/\[br\]/gi, "<br>");
        $(this).html(content);
    });

    // Fix special charaters
    $("textarea").on("paste", function() {
        $textarea = $(this);
        setTimeout(function() {
            content = $textarea.val();
            // ctrl+enter
            var ls = String.fromCharCode(8232);
            find = new RegExp(ls + "(.?)", "g");
            content = content.replace(find, "\n$1");
            // horizontal tab
            var tab = String.fromCharCode(9);
            find = new RegExp(tab, "g");
            content = content.replace(find, " ");
            // trailing spaces
            content = content.replace(/[ |\t]+\n/g, "\n");
            
            $textarea.val(content);
        }, 200);
    });

    // Auto resize textarea height to content
    $("textarea").each(function() { textareaResize($(this), TEXTAREA_MIN); });
    $("textarea").unbind("keyup").keyup(function() { textareaResize($(this), TEXTAREA_MIN); });

    // Fix unchecked radio option
    $("section ul.radio").each(function() {
        if(!$(this).find("input:checked").length) {
            $(this).find("input").first().prop("checked", true);
        };
    });

    // Date update
    $("section .date_box input").unbind("change").change(function() {
        var $section = $(this).closest("section");
        saveDate($section);
        scrollToSorted($section);
    });

    // Date update
    $("section.string input.string").unbind("change").change(function() {
        var $section = $(this).closest("section");
        updateString($section, true);
    });
};

function updateString($section, input_focus) {
    $string = $section.find("input.string");
    var string = $string.val();
    // ====== Get options ======
    var options = $section.find("input.options").val();
    if(options.indexOf("default=") > -1) {
        var default_value = options.split("default=").pop().split(";").shift();
    }
    else {
        var default_value = false;
    };
    if(options.indexOf("type=") > -1) {
        var value_type = options.split("type=").pop().split(";").shift();
    }
    else {
        var value_type = false;
    };
    options = options.split(";");
    // ====== Check for options to apply ======
    // Lowercase/Uppercase/Capitalize
    if(string != "" && options.indexOf("lowercase") > -1) {
        string = string.toLowerCase();
        $string.val(string);
    }
    else if(string != "" && options.indexOf("upercase") > -1) {
        string = string.toUpperCase();
        $string.val(string);
    }
    else if(string != "" && options.indexOf("capitalize") > -1) {
        string = string.capitalize("title");
        $string.val(string);
    };
    //alert("string: " + string + "\noptions: " + options.join(", ") + "\ntype: " + value_type + "\nfocus: " + input_focus);
    // Default
    if(string == "" && default_value && input_focus != true) {
        $string.val(default_value);
    }
    // Required
    else if(string == "" && options.indexOf("required") > -1) {
        if(input_focus == true) {
            infoPopup(LOCALIZE["empty-input-alert"]);
            $string.focus();
        }
    }
    // Type
    else if(string != "" && value_type) {
        if(value_type == "email" && !validateEmail(string)) {
            if(input_focus == true) {
                infoPopup(LOCALIZE["enter-email"]);
                $string.focus();
            }
            else {
                $string.val("");
            }
        }
        else if(value_type == "url" && !validateUrl(string)) {
            if(input_focus == true) {
                infoPopup(LOCALIZE["enter-url"]);
                $string.focus();
            }
            else {
                $string.val("");
            }
        }
        else if(value_type == "tag") {
            if(string.replace(/[^a-z0-9|^\-|^_]/gi, "") != string) {
                if(input_focus == true) {
                    infoPopup(LOCALIZE["enter-tag"]);
                    $string.focus();
                }
                else {
                    $string.val("");
                }
            }
        }
        else if(value_type == "letter" && string.length > 0) {
            if(string.replace(/[^a-z]/gi, "") != string || string.length > 1) {
                if(input_focus == true) {
                    infoPopup(LOCALIZE["enter-letter"]);
                    $string.focus();
                }
                else {
                    $string.val("");
                }
            }
        }
        else if(value_type == "number" && parseFloat(string) != string) {
            if(input_focus == true) {
                infoPopup(LOCALIZE["enter-number"]);
                $string.focus();
            }
            else {
                $string.val("");
            }
        }
        else if(value_type == "integer" && parseInt(string) != string) {
            if(input_focus == true) {
                infoPopup(LOCALIZE["enter-integer"]);
                $string.focus();
            }
            else {
                $string.val("");
            }
        }
        else if(value_type == "float") {
            if(parseInt(string) == string) {
                string = string + ".0";
                $string.val(string);
            }
            else if(parseFloat(string) != string) {
                if(input_focus == true) {
                    infoPopup(LOCALIZE["enter-number"]);
                    $string.focus();
                }
                else {
                    $string.val("");
                }
            }
        }
    }
};

function launchDetectChanges() {
    if(EDIT_PATH.path("extension") == "template" && SAVE_PATH.path("extension") != "template") {
        $("#detectChanges").val("NEW");
    };
    $("form article section.string").each(function() { updateString($(this), false); });
};

// ====== Dynamic changes detector ======
function detectChanges() {
    updateOutput();
    if($("#detectChanges").val() != $("#output").val() + $("#media_descriptions").val()) {
        return true;
    }
    else {
        return false;
    };
};

// ==========================================
//                 Outputs
// ==========================================

// Output test
function updateOutput() {
    $("#style_buttons").remove(); // remove temp content
    if($("#output").attr("name") == "output|order") {
        order = makeOrder();
        $("#output").val(order);
    }
    else {
        xml = makeXml();
        $("#output").val(xml);
        if(typeof(MEDIA_DESCRIPTIONS) == "object" && Object.keys(MEDIA_DESCRIPTIONS).length) {
            var json = arrayToJson(MEDIA_DESCRIPTIONS);
            $("#media_descriptions").val(json);
        }
    };

    // ====== Memorize initial state ======
    if($("#detectChanges").val() == "-") {
        $("#detectChanges").val($("#output").val() + $("#media_descriptions").val());
    };

};

function launchCodePreview() {
    $("#update_output").click(function() {
        $(this).blur();

        updateOutput();
        html = $("#output").val();
        html = html.replace(/<\?(.*?)\?>/g, "[?$1?]");
        html = html.replace(/<(.*?)>/g, "<span class='tag'>&lt;$1&gt;</span>");
        html = html.replace(/\[\?(.*?)\?\]/g, "<span class='flag'>&lt;?$1?&gt;</span>");
        html = html.replace(/\n/g, "<br>");
        html = html.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
        $("#outputs .xml_preview").html(html);

        $("#outputs").find("input").each(function() {
            $group = $(this).prev("p.label").add( $(this) );
            if($(this).val() != "") { $group.show(); }
            else { $group.hide(); };
        })

        if(detectChanges()) {
            $("#outputs .header .changed").text(" (changed)");
        }
        else {
            $("#outputs .header .changed").text("");
        };

        $("#outputs").fadeIn(500);       
        $(document).keyup(function(e) { if (e.keyCode === 27) { hideOutputPreview(); }; });    
        return false;
    });
    
    $("#outputs .close").click(function() {  hideOutputPreview(); return false; });
};

function hideOutputPreview() {
    $("#outputs").fadeOut(250);
    $(document).unbind("keyup");
};

function getVal($id) {
// --------------------------------
// $id = jquery element ID
// --------------------------------
// RETURNS: <string> element text value
// --------------------------------
    tag = $id.prop("tagName").toLowerCase();
    //alert(tag);
    if(tag == "textarea") { return $id.val().replace(/\n/g, "[br]"); }
    else if(tag == "input") { return $id.val(); }
    else if(tag == "p") { return $id.text(); }
    else if(tag == "img" || tag == "iframe" || tag == "embed") { return $id.attr("src"); }
    else { return $id.attr("value"); };
};

function makeXml() {
// --------------------------------
// RETURNS: <string> XML output made from HTML CMS form
// --------------------------------
    //alert("update xml")
    xml = "<xable>\n";
    // Article
    $("#cms.xml article").each(function() {
        article_tag = getItemTag( $(this) );
        if(typeof(article_tag) != "") {
            xml = xml + "\t<" + article_tag + ">\n";
            set = ""; // reset media set
            // Section
            $(this).children("section").each(function() {
                section_tag = getItemTag( $(this) );
                xml = xml + "\t\t<" + section_tag + ">\n";
                // Attributes
                $(this).children().each(function() {
                    var att_tag = getItemTag( $(this) );
                    if(att_tag != "") {
                        // ====== Type ======
                        if(att_tag == "type") {
                            type = $(this).val();
                            xml = xml + "\t\t\t<type>" + type + "</type>\n";
                        }
                        // ====== File ======
                        else if( att_tag == "file" ) {
                            val = getVal( $(this) );
                            xml = xml + "\t\t\t<" + att_tag + ">" + val + "</" + att_tag + ">\n";
                        }
                        // ====== Table ======
                        else if( att_tag == "table" ) {
                            xml = xml + "\t\t\t<table>\n";
                            //alert($(this).html());

                            $(this).children("table").each(function() {

                                language = getItemTag( $(this) );
                                //if(language == "pl") { alert($(this).get(0).outerHTML); };

                                $(this).find("tr").each(function() {

                                    row = [];
                                    num = getItemTag( $(this) );
                                    if(num != "start") {
                                        $(this).find("td").each(function() {
                                            // Strip off all formating except <br>
                                            val = $(this).html().replace(/\&nbsp;/g, " ").replace(/<br>/g, "[br]");
                                            $(this).html(val); // put back modified content
                                            val = $(this).text(); // get pure text
                                            $(this).html(val.replace(/\[br\]/g, "<br>")); // RESTORE <BR>
                                            // XML outupt
                                            val = val.replace(/;/g, "[semi]"); // change semicolons to bbcode
                                            row[row.length] = val;
                                        });
                                        row.shift(); // delete row numbers
                                        xml = xml + "\t\t\t\t<" + language + ">" + row.join(";") + "</" + language + ">\n";
                                    };
                                });
                            });
                            xml = xml + "\t\t\t</table>\n";
                        }
                        // ====== media Set ======
                        else if( att_tag == "set" ) {
                            mode = $(this).find("input").attr("type");
                            set = $(this).find("input").first().val(); // fix nothing selected
                            $(this).find("input").each(function() {
                                if( $(this).prop("checked") == true ) { set = $(this).val(); };
                            });
                        }
                        // ====== Complex data / <i></i> tag will be ignored ======
                        else if( $(this).children().length && $(this).children().prop("tagName") != "I") {
                            xml = xml + "\t\t\t<" + att_tag + ">\n";

                            $(this).children().each(function() {
                                children_tag = getItemTag( $(this) );
                                children_val = getVal( $(this) );

                                if((children_tag == "" || children_val == undefined) && $(this).children().length) {
                                    children_tag = getItemTag( $(this).children() );
                                    children_val = getVal( $(this).children() );
                                };
                                if(att_tag == "text") {
                                    children_val = children_val.replace(/\n/g, "[br]"); // enter
                                    children_val = children_val.replace(/–/g, "-"); // dash -> minus
                                    children_val = children_val.replace(/"/g, "&quot;"); // quotes
                                    children_val = children_val.replace(/</g, "&lt;").replace(/>/g, "&gt;"); // html tags & comments
                                }; // text fix
                                xml = xml + "\t\t\t\t<" + children_tag + ">" + children_val + "</" + children_tag + ">\n";
                            });
                            xml = xml + "\t\t\t</" + att_tag + ">\n";
                            if(type == "media") {
                                xml = xml + "\t\t\t<set>" + set + "</set>\n";
                            };
                            // ====== options Selected ======
                            if(type == "option") {
                                selected = [];
                                $(this).closest("section").find(".option input").each(function() {
                                    if( $(this).prop("checked") == true ) { selected[selected.length] = $(this).val(); };
                                });
                                if(selected.length > 0) { selected = selected.join(";"); } else { selected = ""; }; 
                                xml = xml + "\t\t\t<selected>" + selected + "</selected>\n";
                            };
                        }
                        // ====== Simple data ======
                        else {
                            att_val = getVal( $(this) );
                            att_val = att_val.replace(/</g, "&lt;").replace(/>/g, "&gt;"); // html tags & comments
                            //if(type == "code") { att_val = att_val.replace(/\[/g, "&#91;").replace(/\]/g, "&#93;"); };
                            //alert(att_tag + " -> " + att_val);
                            xml = xml + "\t\t\t<" + att_tag + ">" + att_val + "</" + att_tag + ">\n";
                        };
                    };
                });
                xml = xml + "\t\t</" + section_tag + ">\n";
            });
            xml = xml + "\t</" + article_tag + ">\n";
        };
    });
    // ====== OUTPUT ======
    xml = xml + "</xable>\n";
    //alert("xml done")
    return xml;
};

function makeOrder() {
    order = [];
    order[order.length] = "<_order>\n" +
        "\t<title>" + $("#cms.order input._order-title").val() +"</title>\n" +
        "\t<options>" + getOrderOptions().join(";") +"</options>\n" +
        "</_order>";
    $("#cms.order h3").each(function() {
        item_xml = $(this).find(".value").html().replace(/\&lt;/g, "<").replace(/\&gt;/g, ">");
        order[order.length] = "<multi_item>\n" + item_xml + "</multi_item>";
    });
    return order.join("\n");
};